<?php

namespace TDevAgency\Erp\Controllers;

use Exception;
use Illuminate\Http\RedirectResponse;
use Illuminate\Http\Response;
use Illuminate\Support\Facades\App;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\File;
use Illuminate\Support\Facades\Redirect;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Facades\View;
use Illuminate\Validation\ValidationException;
use TDevAgency\Cms\Controllers\Controller;
use Illuminate\Http\Request;
use TDevAgency\Erp\Models\Customer;
use TDevAgency\Erp\Models\CustomerAddress;
use TDevAgency\Erp\Models\Order;
use TDevAgency\Erp\Models\OrderCustomer;
use TDevAgency\Erp\Models\OrderDelivery;
use TDevAgency\Erp\Models\OrderItem;
use TDevAgency\Erp\Models\OrderPayment;
use TDevAgency\Erp\Models\Product;
use TDevAgency\Erp\Models\Manufacture;
use TDevAgency\NovaPoshtaSdk\Api\AddressGeneral;
use TDevAgency\NovaPoshtaSdk\Facades\NovaPoshta;

class OrderController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Contracts\View\View
     */
    public function index()
    {
        $view = View::make('erp::pages.orders.index');

        return $view->with('orders', Order::container()->sortable()->search()->with(['order_customer', 'order_items', 'order_delivery', 'order_payment'])->paginate(20));
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Contracts\View\View
     */
    public function create()
    {
        $view = View::make('erp::pages.orders.single');
        $formRoute = route('cms.orders.store');
        $formMethod = 'POST';
        $order = new Order();
        return $view->with(compact('formMethod', 'formRoute', 'order'));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param Request $request
     * @return RedirectResponse
     * @throws ValidationException
     */
    public function store(Request $request)
    {
        $this->validate($request, [
            'order_status' => 'required',
            'total_price' => 'required',
            'products_price' => 'required',
            'order_items.*product_id' => 'required'
        ]);

        DB::transaction(function () use ($request) {
            $order = new Order();
            foreach ($order->getFillable() as $field) {
                if ($request->has($field)) {
                    $order->setAttribute($field, $request->get($field));
                }
            }
            // TODO create new customer

            $order->save();
            $order->order_number = 10000 + $order->id;
            $order->update();

            foreach ($request->get('order_items') as $item) {
                $product = Product::find($item['product_id']);
                $orderItem = new OrderItem();
                $orderItem->order_id = $order->id;
                $orderItem->product_id = $product->id;
                $orderItem->product_name = $product->product_name;
                $orderItem->product_sku = $product->sku;
                $orderItem->product_qty = $item['product_qty'];
                $orderItem->price = $item['price'];
                $orderItem->price_discount = 0;
                $orderItem->save();
            }

            $orderPayment = new OrderPayment;
            $orderPayment->order_id = $order->id;
            $orderPayment->payment_status = 0;
            $orderPayment->payment_code = 'cash';
            $orderPayment->payment_name = 'Наличный платеж';
            $orderPayment->save();

            $orderDelivery = new OrderDelivery();
            $orderDelivery->order_id = $order->id;
            $orderDelivery->delivery_status = 0;
            $orderDelivery->delivery_code = $request->get('order_delivery')['delivery_code'] ?? 'pickup';
            $orderDelivery->delivery_name = isset($request->get('order_delivery')['delivery_code']) && $request->get('order_delivery')['delivery_code'] === 'nova_poshta' ? "Новая почта" : 'Самовывоз';
            $orderDelivery->delivery_price = 0;
            $orderDelivery->track_number = $request->get('order_delivery')['track_number'] ?? null;

            if ($orderDelivery->delivery_code === 'nova_poshta') {
                $deliveryData = $request->get('order_delivery')['delivery_data'];
                $deliveryData['delivery_formatted'] = $this->deliveryFormatted($deliveryData);
                $orderDelivery->delivery_data = base64_encode(serialize($deliveryData));
            }
            $orderDelivery->save();

            $customer = $request->get('order_customer');
            $orderCustomer = new OrderCustomer();

            $orderCustomer->order_id = $order->id;
            $orderCustomer->email = $customer['email'] ?? null;
            $orderCustomer->last_name = $customer['last_name'] ?? null;
            $orderCustomer->first_name = $customer['first_name'] ?? null;
            $orderCustomer->city = $customer['city'] ?? null;
            $orderCustomer->street = $customer['street'] ?? null;
            $orderCustomer->house_number = $customer['house_number'] ?? null;
            $orderCustomer->postal_code = $customer['postal_code'] ?? null;
            $orderCustomer->apt_number = $customer['apt_number'] ?? null;
            $orderCustomer->phone_one = $customer['phone_one'] ?? null;

            $orderCustomer->save();


            $this->saveCustomerContact($customer);

        });

        return Redirect::route('cms.orders.index');
    }

    private function deliveryFormatted($data)
    {
        $deps = NovaPoshta::addressGeneralModel()
            ->getWarehouses(['SettlementRef' => $data['city_ref']], AddressGeneral::CATEGORY_BRANCH)->toArray();

        $key = array_search($data['wire_house_ref'], array_column($deps, 'Ref'));

        if (!$key) {
            return null;
        }
        return sprintf('%s, %s.', $deps[$key]['DescriptionRu'], $deps[$key]['CityDescriptionRu']);
    }

    private function saveCustomerContact($customer)
    {
        if ($customer['phone_one'] === null) {
            return true;
        }

        $customerExists = Customer::where('phone', 'like', "%" . $customer['phone_one'] . "%")->exists();
        $customerAddressExists = CustomerAddress::where('phone_one', 'like', "%" . $customer['phone_one'] . "%")->exists();

        if ($customerExists || $customerAddressExists) {
            return true;
        }


        $customerModel = new Customer();

        $customerModel->email = $customer['email'] ?? null;
        $customerModel->phone = $customer['phone_one'] ?? null;
        $customerModel->first_name = $customer['first_name'] ?? null;
        $customerModel->last_name = $customer['last_name'] ?? null;

        $customerModel->save();

        $customerAddress = new CustomerAddress();

        $customerAddress->customer_id = $customerModel->id;
        $customerAddress->phone_one = $customer['phone_one'] ?? null;
        $customerAddress->first_name = $customer['first_name'] ?? null;
        $customerAddress->last_name = $customer['last_name'] ?? null;
        $customerAddress->city = $customer['city'] ?? null;
        $customerAddress->street = $customer['street'] ?? null;
        $customerAddress->house_number = $customer['house_number'] ?? null;
        $customerAddress->apt_number = $customer['apt_number'] ?? null;
        $customerAddress->postal_code = $customer['postal_code'] ?? null;
        $customerAddress->save();
    }

    /**
     * Display the specified resource.
     *
     * @param Manufacture $manufacture
     * @return void
     */
    public function show(Manufacture $manufacture)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param $id
     * @return \Illuminate\Contracts\View\View
     */
    public function edit($id)
    {
        $view = View::make('erp::pages.orders.single');
        $formRoute = route('cms.orders.update', $id);
        $formMethod = 'PUT';
        $order = Order::where('id', $id)->with('order_customer', 'order_items', 'order_delivery')->first();
        return $view->with(compact('formMethod', 'formRoute', 'order'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param Request $request
     * @param $id
     * @return RedirectResponse
     */
    public function update(Request $request, $id)
    {
        $this->validate($request, [
            'order_status' => 'required',
            'total_price' => 'required',
            'products_price' => 'required',
            'order_items.*product_id' => 'required'
        ]);

        DB::transaction(function () use ($request, $id) {
            $order = Order::find($id);
            foreach ($order->getFillable() as $field) {
                if ($request->has($field)) {
                    $order->setAttribute($field, $request->get($field));
                }
            }
            $order->update();

            foreach ($request->get('order_items') as $item) {

                $product = Product::find($item['product_id']);

                OrderItem::updateOrCreate([
                    'id' => $item['id'],
                    'order_id' => $order->id
                ], [
                    'id' => $item['id'],
                    'order_id' => $order->id,
                    'product_id' => $product->id,
                    'product_name' => $product->product_name,
                    'product_sku' => $product->sku,
                    'product_qty' => $item['product_qty'],
                    'price' => $item['price'],
                ]);
            }

            $customer = $request->get('order_customer');
            OrderCustomer::updateOrCreate([
                'order_id' => $order->id
            ], [
                'order_id' => $order->id,
                'email' => $customer['email'] ?? null,
                'last_name' => $customer['last_name'] ?? null,
                'first_name' => $customer['first_name'] ?? null,
                'city' => $customer['city'] ?? null,
                'street' => $customer['street'] ?? null,
                'house_number' => $customer['house_number'] ?? null,
                'postal_code' => $customer['postal_code'] ?? null,
                'apt_number' => $customer['apt_number'] ?? null,
                'phone_one' => $customer['phone_one'] ?? null,
            ]);

            $this->saveCustomerContact($customer);


            $order_delivery = $request->get('order_delivery');
            $order_delivery_params = [
                'track_number' => $order_delivery['track_number'] ?? null,
                'delivery_code' => $order_delivery['delivery_code'] ?? 'pickup',
                'delivery_name' => isset($order_delivery['delivery_code']) && $order_delivery['delivery_code'] === 'nova_poshta' ? "Новая почта" : 'Самовывоз'
            ];
            if (isset($order_delivery['delivery_data']) && !empty($order_delivery['delivery_data'])) {

                $deliveryData = $order_delivery['delivery_data'];
                $deliveryData['delivery_formatted'] = $this->deliveryFormatted($deliveryData);
                $order_delivery_params['delivery_data'] = base64_encode(serialize($deliveryData));
            }
            OrderDelivery::updateOrCreate([
                'order_id' => $order->id
            ], $order_delivery_params);

        });

        return Redirect::route('cms.orders.index');
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param $id
     * @return RedirectResponse
     * @throws Exception
     */
    public function destroy($id)
    {
    }
}
